@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.26.2.10 2009/11/14 13:16:49 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/child.c,v 1.26.2.13 2010/04/13 04:59:12 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2009	PgPool Global Development Group
+ * Copyright (c) 2003-2010	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -101,7 +101,6 @@ void do_child(int unix_fd, int inet_fd)
 	struct timeval timeout;
 	static int connected;
 	int connections_count = 0;	/* used if child_max_connections > 0 */
-	int first_ready_for_query_received;		/* for master/slave mode */
 	int found;
 	char psbuf[NI_MAXHOST + 128];
 
@@ -293,8 +292,6 @@ void do_child(int unix_fd, int inet_fd)
 		 * we need to connect to the backend and send the startup packet.
 		 */
 
-		first_ready_for_query_received = 0;		/* for master/slave mode */
-
 		/* look for existing connection */
 		found = 0;
 		backend = pool_get_cp(sp->user, sp->database, sp->major, 1);
@@ -336,13 +333,6 @@ void do_child(int unix_fd, int inet_fd)
 				connection_count_down();
 				continue;
 			}
-
-			/* in master/slave mode, the first "ready for query"
-			 * packet should be treated as if we were not in the
-			 * mode
-			 */
-			if (MASTER_SLAVE)
-				first_ready_for_query_received = 1;
 		}
 
 		else
@@ -429,7 +419,7 @@ void do_child(int unix_fd, int inet_fd)
 		{
 			POOL_STATUS status;
 
-			status = pool_process_query(frontend, backend, 0, first_ready_for_query_received);
+			status = pool_process_query(frontend, backend, 0);
 
 			sp = MASTER_CONNECTION(backend)->sp;
 
@@ -458,7 +448,7 @@ void do_child(int unix_fd, int inet_fd)
 						POOL_STATUS status1;
 
 						/* send reset request to backend */
-						status1 = pool_process_query(frontend, backend, 1, 0);
+						status1 = pool_process_query(frontend, backend, 1);
 						pool_close(frontend);
 
 						/* if we detect errors on resetting connection, we need to discard
@@ -1095,22 +1085,23 @@ void cancel_request(CancelPacket *sp)
 	int i;
 	ConnectionInfo *c = NULL;
 	CancelPacket cp;
+	int loop_num = pool_config->num_init_children*pool_config->max_pool*MAX_NUM_BACKENDS;
 
 	pool_debug("Cancel request received");
 
 	/* look for cancel key from shmem info */
-	for (i=0;i<pool_config->num_init_children*pool_config->max_pool;i++)
+	for (i=0;i<loop_num;i++)
 	{
 		c = &con_info[i];
 
 		if (c->pid == sp->pid && c->key == sp->key)
 		{
 			pool_debug("found pid:%d key:%d i:%d",c->pid, c->key,i);
-			c = &con_info[i/pool_config->max_pool * pool_config->max_pool];
+			c = &con_info[i/(pool_config->max_pool * pool_config->max_pool)];
 			break;
 		}
 	}
-	if (i == pool_config->num_init_children*pool_config->max_pool)
+	if (i == loop_num)
 		return;	/* invalid key */
 
 	for (i=0;i<NUM_BACKENDS;i++,c++)
@@ -1416,13 +1407,13 @@ POOL_CONNECTION_POOL_SLOT *make_persistent_db_connection(
 	int len, len1;
 	int status;
 
-	cp = malloc(sizeof(POOL_CONNECTION_POOL));
+	cp = malloc(sizeof(POOL_CONNECTION_POOL_SLOT));
 	if (cp == NULL)
 	{
 		pool_error("make_persistent_db_connection: could not allocate memory");
 		return NULL;
 	}
-	memset(cp, 0, sizeof(POOL_CONNECTION_POOL));
+	memset(cp, 0, sizeof(POOL_CONNECTION_POOL_SLOT));
 
 	startup_packet = malloc(sizeof(*startup_packet));
 	if (startup_packet == NULL)
