@@ -1,8 +1,26 @@
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pcp/pcp.c,v 1.2.2.1 2007/08/29 06:40:59 y-asaba Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pcp/pcp.c,v 1.8.2.1 2010/01/09 00:51:12 t-ishii Exp $
  *
  * Handles PCP connection, and protocol communication with pgpool-II
  * These are client APIs. Server program should use APIs in pcp_stream.c
+ *
+ *
+ * pgpool: a language independent connection pool server for PostgreSQL 
+ * written by Tatsuo Ishii
+ *
+ * Copyright (c) 2003-2008	PgPool Global Development Group
+ *
+ * Permission to use, copy, modify, and distribute this software and
+ * its documentation for any purpose and without fee is hereby
+ * granted, provided that the above copyright notice appear in all
+ * copies and that both that copyright notice and this permission
+ * notice appear in supporting documentation, and that the name of the
+ * author not be used in advertising or publicity pertaining to
+ * distribution of the software without specific, written prior
+ * permission. The author makes no representations about the
+ * suitability of this software for any purpose.  It is provided "as
+ * is" without express or implied warranty.
+ *
  */
 
 #include <stdio.h>
@@ -24,6 +42,11 @@
 struct timeval pcp_timeout;
 
 static PCP_CONNECTION *pc;
+#ifdef DEBUG
+static int debug = 1;
+#else
+static int debug = 0;
+#endif
 static int pcp_authorize(char *username, char *password);
 
 /* --------------------------------
@@ -44,9 +67,7 @@ pcp_connect(char *hostname, int port, char *username, char *password)
 
 	if (pc != NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection to backend \"%s\" already exists\n", hostname);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection to backend \"%s\" already exists\n", hostname);
 		return 0;
 	}
 
@@ -58,9 +79,7 @@ pcp_connect(char *hostname, int port, char *username, char *password)
 
 		if (fd < 0)
 		{
-#ifdef DEBUG
-			fprintf(stderr, "DEBUG: could not create socket\n");
-#endif
+			if (debug) fprintf(stderr, "DEBUG: could not create socket\n");
 			errorcode = SOCKERR;
 			return -1;
 		}
@@ -82,9 +101,7 @@ pcp_connect(char *hostname, int port, char *username, char *password)
 
 		if (connect(fd, (struct sockaddr *) &unix_addr, sizeof(unix_addr)) < 0)
 		{
-#ifdef DEBUG
-			fprintf(stderr, "DEBUG: could not connect to \"%s\"\n", unix_addr.sun_path);
-#endif
+			if (debug) fprintf(stderr, "DEBUG: could not connect to \"%s\"\n", unix_addr.sun_path);
 			close(fd);
 			errorcode = CONNERR;
 			return -1;
@@ -95,9 +112,7 @@ pcp_connect(char *hostname, int port, char *username, char *password)
 		fd = socket(AF_INET, SOCK_STREAM, 0);
 		if (fd < 0)
 		{
-#ifdef DEBUG
-			fprintf(stderr, "DEBUG: could not create socket\n");
-#endif
+		  	if (debug) fprintf(stderr, "DEBUG: could not create socket\n");
 			errorcode = SOCKERR;
 			return -1;
 		}
@@ -105,9 +120,7 @@ pcp_connect(char *hostname, int port, char *username, char *password)
 		if (setsockopt(fd, IPPROTO_TCP, TCP_NODELAY,
 					   (char *) &on, sizeof(on)) < 0)
 		{
-#ifdef DEBUG
-			fprintf(stderr, "DEBUG: could not set socket option\n");
-#endif
+			if (debug) fprintf(stderr, "DEBUG: could not set socket option\n");
 			close(fd);
 			errorcode = SOCKERR;
 			return -1;
@@ -118,9 +131,7 @@ pcp_connect(char *hostname, int port, char *username, char *password)
 		hp = gethostbyname(hostname);
 		if ((hp == NULL) || (hp->h_addrtype != AF_INET))
 		{
-#ifdef DEBUG
-			fprintf(stderr, "DEBUG: could not retrieve hostname\n");
-#endif
+			if (debug) fprintf(stderr, "DEBUG: could not retrieve hostname\n");
 			close(fd);
 			errorcode = HOSTERR;
 			return -1;
@@ -133,9 +144,7 @@ pcp_connect(char *hostname, int port, char *username, char *password)
 		len = sizeof(struct sockaddr_in);
 		if (connect(fd, (struct sockaddr *) &addr, len) < 0)
 		{
-#ifdef DEBUG
-			fprintf(stderr, "DEBUG: could not connect to \"%s\"\n", hostname);
-#endif
+			if (debug) fprintf(stderr, "DEBUG: could not connect to \"%s\"\n", hostname);
 			close(fd);
 			errorcode = CONNERR;
 			return -1;
@@ -145,9 +154,7 @@ pcp_connect(char *hostname, int port, char *username, char *password)
 	pc = pcp_open(fd);
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not allocate buffer space\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not allocate buffer space\n");
 		close(fd);
 		return -1;
 	}
@@ -184,9 +191,7 @@ pcp_authorize(char *username, char *password)
 	pcp_write(pc, &wsize, sizeof(int));
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return -1;
 	}
 
@@ -225,14 +230,10 @@ pcp_authorize(char *username, char *password)
 	pcp_write(pc, encrypt_buf, strlen(encrypt_buf)+1);
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if  (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return -1;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"R\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"R\", len=%d\n", ntohl(wsize));
 
 	if (pcp_read(pc, &tos, 1))
 		return -1;
@@ -247,15 +248,11 @@ pcp_authorize(char *username, char *password)
 	}
 	if (pcp_read(pc, buf, rsize - sizeof(int)))
 		return -1;
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
-#endif
+	if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
 
 	if (tos == 'e')
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
 		errorcode = BACKENDERR;
 	}
 	else if (tos == 'r')
@@ -266,9 +263,7 @@ pcp_authorize(char *username, char *password)
 			return 0;
 		}
 
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: authentication failed. reason=%s\n", buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: authentication failed. reason=%s\n", buf);
 		errorcode = AUTHERR;
 	}
 	free(buf);
@@ -287,9 +282,7 @@ pcp_disconnect(void)
 
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		return;
 	}
 
@@ -300,9 +293,7 @@ pcp_disconnect(void)
 	{
 		/* backend had closed connection already */
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"X\", len=%d\n", sizeof(int));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"X\", len=%zd\n", sizeof(int));
 
 	pcp_close(pc);
 	pc = NULL;
@@ -321,9 +312,7 @@ pcp_terminate_pgpool(char mode)
 
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		errorcode = NOCONNERR;
 		return -1;
 	}
@@ -334,14 +323,10 @@ pcp_terminate_pgpool(char mode)
 	pcp_write(pc, &mode, sizeof(char));
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return -1;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"T\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"T\", len=%d\n", ntohl(wsize));
 
 	return 0;
 }
@@ -363,9 +348,7 @@ pcp_node_count(void)
 
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		errorcode = NOCONNERR;
 		return -1;
 	}
@@ -375,14 +358,10 @@ pcp_node_count(void)
 	pcp_write(pc, &wsize, sizeof(int));
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return -1;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"L\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"L\", len=%d\n", ntohl(wsize));
 
 	if (pcp_read(pc, &tos, 1))
 		return -1;
@@ -401,22 +380,18 @@ pcp_node_count(void)
 		return -1;
 	}
 
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
-#endif
+	if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
 
 	if (tos == 'e')
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
 		errorcode = BACKENDERR;
 	}
 	else if (tos == 'l')
 	{
 		if (strcmp(buf, "CommandComplete") == 0)
 		{
-			index = memchr(buf, '\0', rsize) + 1;
+			index = (char *) memchr(buf, '\0', rsize) + 1;
 			if (index != NULL)
 			{
 				int ret = atoi(index);
@@ -448,9 +423,7 @@ pcp_node_info(int nid)
 
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		errorcode = NOCONNERR;
 		return NULL;
 	}
@@ -463,14 +436,10 @@ pcp_node_info(int nid)
 	pcp_write(pc, node_id, strlen(node_id)+1);
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return NULL;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"I\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"I\", len=%d\n", ntohl(wsize));
 
 	if (pcp_read(pc, &tos, 1))
 		return NULL;	
@@ -489,15 +458,11 @@ pcp_node_info(int nid)
 		return NULL;
 	}
 
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
-#endif
+	if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
 
 	if (tos == 'e')
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
 		errorcode = BACKENDERR;
 		free(buf);
 		return NULL;
@@ -519,19 +484,19 @@ pcp_node_info(int nid)
 			// FIXME
 //			rsize -= strlen("CommandComplete") + 1;
 
- 			index = memchr(buf, '\0', rsize) + 1;
+ 			index = (char *) memchr(buf, '\0', rsize) + 1;
 			if (index != NULL)
 				strcpy(backend_info->backend_hostname, index);
 
-			index = memchr(index, '\0', rsize) + 1;
+			index = (char *) memchr(index, '\0', rsize) + 1;
 			if (index != NULL)
 				backend_info->backend_port = atoi(index);
 
-			index = memchr(index, '\0', rsize) + 1;
+			index = (char *) memchr(index, '\0', rsize) + 1;
 			if (index != NULL)
 				backend_info->backend_status = atoi(index);
 
-			index = memchr(index, '\0', rsize) + 1;
+			index = (char *) memchr(index, '\0', rsize) + 1;
 			if (index != NULL)
 				backend_info->backend_weight = atof(index);
 
@@ -560,9 +525,7 @@ pcp_process_count(int *pnum)
 	
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		errorcode = NOCONNERR;
 		return NULL;
 	}
@@ -572,14 +535,10 @@ pcp_process_count(int *pnum)
 	pcp_write(pc, &wsize, sizeof(int));
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return NULL;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"N\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"N\", len=%d\n", ntohl(wsize));
 
 	if (pcp_read(pc, &tos, 1))
 		return NULL;			
@@ -597,15 +556,11 @@ pcp_process_count(int *pnum)
 		free(buf);
 		return NULL;		
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
-#endif
+	if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
 
 	if (tos == 'e')
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
 		free(buf);
 		errorcode = BACKENDERR;
 		return NULL;
@@ -619,7 +574,7 @@ pcp_process_count(int *pnum)
 			char *index = NULL;
 			int i;
 
-			index = memchr(buf, '\0', rsize) + 1;
+			index = (char *) memchr(buf, '\0', rsize) + 1;
 			process_count = atoi(index);
 
 			process_list = (int *)malloc(sizeof(int) * process_count);
@@ -632,7 +587,7 @@ pcp_process_count(int *pnum)
 
 			for (i = 0; i < process_count; i++)
 			{
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				process_list[i] = atoi(index);
 			}
 
@@ -667,9 +622,7 @@ pcp_process_info(int pid, int *array_size)
 
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		errorcode = NOCONNERR;
 		return NULL;
 	}
@@ -682,14 +635,10 @@ pcp_process_info(int pid, int *array_size)
 	pcp_write(pc, process_id, strlen(process_id)+1);
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return NULL;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"P\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"P\", len=%d\n", ntohl(wsize));
 
 	while (1)
 	{
@@ -709,15 +658,11 @@ pcp_process_info(int pid, int *array_size)
 			free(buf);
 			return NULL;
 		}
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
 
 		if (tos == 'e')
 		{
-#ifdef DEBUG
-			fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
-#endif
+			if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
 			free(buf);
 			errorcode = BACKENDERR;
 			return NULL;
@@ -728,7 +673,7 @@ pcp_process_info(int pid, int *array_size)
 
 			if (strcmp(buf, "ArraySize") == 0)
 			{
-				index = memchr(buf, '\0', rsize) + 1;
+				index = (char *) memchr(buf, '\0', rsize) + 1;
 				if (index != NULL)
 					ci_size = atoi(index);
 
@@ -754,31 +699,31 @@ pcp_process_info(int pid, int *array_size)
 			}
 			else if (strcmp(buf, "ProcessInfo") == 0)
 			{
-				index = memchr(buf, '\0', rsize) + 1;
+				index = (char *) memchr(buf, '\0', rsize) + 1;
 				if (index != NULL)
 					strcpy(process_info->connection_info[offset].database, index);
 			
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					strcpy(process_info->connection_info[offset].user, index);
 			
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					process_info->start_time = atol(index);
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					process_info->connection_info[offset].create_time = atol(index);
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					process_info->connection_info[offset].major = atoi(index);
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					process_info->connection_info[offset].minor = atoi(index);
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					process_info->connection_info[offset].counter = atoi(index);
 
@@ -818,9 +763,7 @@ pcp_systemdb_info(void)
 
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		errorcode = NOCONNERR;
 		return NULL;
 	}
@@ -830,14 +773,10 @@ pcp_systemdb_info(void)
 	pcp_write(pc, &wsize, sizeof(int));
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return NULL;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"S\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"S\", len=%d\n", ntohl(wsize));
 
 	while (1) {
 		if (pcp_read(pc, &tos, 1))
@@ -856,15 +795,11 @@ pcp_systemdb_info(void)
 			free(buf);
 			return NULL;
 		}
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
 
 		if (tos == 'e')
 		{
-#ifdef DEBUG
-			fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
-#endif
+			if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
 			free(buf);
 			errorcode = BACKENDERR;
 			return NULL;
@@ -883,7 +818,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 
-				index = memchr(buf, '\0', rsize) + 1;
+				index = (char *) memchr(buf, '\0', rsize) + 1;
 				if (index != NULL)
 					systemdb_info->hostname = strdup(index);
 				if (systemdb_info->hostname == NULL)
@@ -894,11 +829,11 @@ pcp_systemdb_info(void)
 					return NULL;
 				}				
 			
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					systemdb_info->port = atoi(index);
 			
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					systemdb_info->user = strdup(index);
 				if (systemdb_info->user == NULL)
@@ -909,7 +844,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					systemdb_info->password = strdup(index);
 				if (systemdb_info->password == NULL)
@@ -920,7 +855,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					systemdb_info->schema_name = strdup(index);
 				if (systemdb_info->schema_name == NULL)
@@ -931,7 +866,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					systemdb_info->database_name = strdup(index);
 				if (systemdb_info->database_name == NULL)
@@ -942,11 +877,11 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 			
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					systemdb_info->dist_def_num = atoi(index);
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					systemdb_info->system_db_status = atoi(index);
 
@@ -977,7 +912,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 
-				index = memchr(buf, '\0', rsize) + 1;
+				index = (char *) memchr(buf, '\0', rsize) + 1;
 				if (index != NULL)
 					dist_def_info->dbname = strdup(index);
 				if (dist_def_info->dbname == NULL)
@@ -988,7 +923,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 			
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					dist_def_info->schema_name = strdup(index);
 				if (dist_def_info->schema_name == NULL)
@@ -999,7 +934,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 			
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					dist_def_info->table_name = strdup(index);
 				if (dist_def_info->table_name == NULL)
@@ -1010,7 +945,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					dist_def_info->dist_key_col_name = strdup(index);
 				if (dist_def_info->dist_key_col_name == NULL)
@@ -1021,7 +956,7 @@ pcp_systemdb_info(void)
 					return NULL;
 				}
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					dist_def_info->col_num = atoi(index);
 
@@ -1036,7 +971,7 @@ pcp_systemdb_info(void)
 				}
 				for (i = 0; i < dist_def_info->col_num; i++)
 				{
-					index = memchr(index, '\0', rsize) + 1;
+					index = (char *) memchr(index, '\0', rsize) + 1;
 					if (index != NULL)
 						dist_def_info->col_list[i] = strdup(index);
 					if (dist_def_info->col_list[i] == NULL)
@@ -1059,7 +994,7 @@ pcp_systemdb_info(void)
 				}
 				for (i = 0; i < dist_def_info->col_num; i++)
 				{
-					index = memchr(index, '\0', rsize) + 1;
+					index = (char *) memchr(index, '\0', rsize) + 1;
 					if (index != NULL)
 						dist_def_info->type_list[i] = strdup(index);
 					if (dist_def_info->type_list[i] == NULL)
@@ -1071,7 +1006,7 @@ pcp_systemdb_info(void)
 					}
 				}
 
-				index = memchr(index, '\0', rsize) + 1;
+				index = (char *) memchr(index, '\0', rsize) + 1;
 				if (index != NULL)
 					dist_def_info->dist_def_func = strdup(index);
 				if (dist_def_info->dist_def_func == NULL)
@@ -1148,9 +1083,7 @@ pcp_detach_node(int nid)
 
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		errorcode = NOCONNERR;
 		return -1;
 	}
@@ -1163,14 +1096,10 @@ pcp_detach_node(int nid)
 	pcp_write(pc, node_id, strlen(node_id)+1);
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return -1;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"D\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"D\", len=%d\n", ntohl(wsize));
 
 	if (pcp_read(pc, &tos, 1))
 		return -1;
@@ -1188,15 +1117,11 @@ pcp_detach_node(int nid)
 		free(buf);
 		return -1;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
-#endif
+	if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
 
 	if (tos == 'e')
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
 		errorcode = BACKENDERR;
 	}
 	else if (tos == 'd')
@@ -1231,9 +1156,7 @@ pcp_attach_node(int nid)
 
 	if (pc == NULL)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: connection does not exist\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
 		errorcode = NOCONNERR;
 		return -1;
 	}
@@ -1246,14 +1169,10 @@ pcp_attach_node(int nid)
 	pcp_write(pc, node_id, strlen(node_id)+1);
 	if (pcp_flush(pc) < 0)
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: could not send data to backend\n");
-#endif
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
 		return -1;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: send: tos=\"D\", len=%d\n", ntohl(wsize));
-#endif
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"D\", len=%d\n", ntohl(wsize));
 
 	if (pcp_read(pc, &tos, 1))
 		return -1;
@@ -1271,15 +1190,11 @@ pcp_attach_node(int nid)
 		free(buf);
 		return -1;
 	}
-#ifdef DEBUG
-	fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
-#endif
+	if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
 
 	if (tos == 'e')
 	{
-#ifdef DEBUG
-		fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
-#endif
+		if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
 		errorcode = BACKENDERR;
 	}
 	else if (tos == 'c')
@@ -1299,5 +1214,86 @@ pcp_attach_node(int nid)
 void
 pcp_set_timeout(long sec)
 {
+	/* disable timeout (wait forever!) (2008/02/08 yamaguti) */
+	sec = 0;
 	pcp_timeout.tv_sec = sec;
 }
+
+
+int
+pcp_recovery_node(int nid)
+{
+	int wsize;
+	char node_id[16];
+	char tos;
+	char *buf = NULL;
+	int rsize;
+
+	if (pc == NULL)
+	{
+		if (debug) fprintf(stderr, "DEBUG: connection does not exist\n");
+		errorcode = NOCONNERR;
+		return -1;
+	}
+
+	snprintf(node_id, sizeof(node_id), "%d", nid);
+
+	pcp_write(pc, "O", 1);
+	wsize = htonl(strlen(node_id)+1 + sizeof(int));
+	pcp_write(pc, &wsize, sizeof(int));
+	pcp_write(pc, node_id, strlen(node_id)+1);
+	if (pcp_flush(pc) < 0)
+	{
+		if (debug) fprintf(stderr, "DEBUG: could not send data to backend\n");
+		return -1;
+	}
+	if (debug) fprintf(stderr, "DEBUG: send: tos=\"D\", len=%d\n", ntohl(wsize));
+
+	if (pcp_read(pc, &tos, 1))
+		return -1;
+	if (pcp_read(pc, &rsize, sizeof(int)))
+		return -1;
+	rsize = ntohl(rsize);
+	buf = (char *)malloc(rsize);
+	if (buf == NULL)
+	{
+		errorcode = NOMEMERR;
+		return -1;
+	}
+	if (pcp_read(pc, buf, rsize - sizeof(int)))
+	{
+		free(buf);
+		return -1;
+	}
+	if (debug) fprintf(stderr, "DEBUG: recv: tos=\"%c\", len=%d, data=%s\n", tos, rsize, buf);
+
+	if (tos == 'e')
+	{
+		if (debug) fprintf(stderr, "DEBUG: command failed. reason=%s\n", buf);
+		errorcode = BACKENDERR;
+	}
+	else if (tos == 'c')
+	{
+		/* strcmp() for success message, or fail */
+		if(strcmp(buf, "CommandComplete") == 0)
+		{
+			free(buf);
+			return 0;
+		}
+	}
+
+	free(buf);
+	return -1;
+}
+
+void
+pcp_enable_debug(void)
+{
+	debug = 1;
+}
+
+void
+pcp_disable_debug(void)
+{
+	debug = 0;
+}
