@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_connection_pool.c,v 1.13.2.3 2009/08/22 04:19:49 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_connection_pool.c,v 1.13.2.5 2010/04/13 04:59:12 t-ishii Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
@@ -69,7 +69,7 @@ int pool_init_cp(void)
 	for (i = 0; i < pool_config->max_pool; i++)
 	{
 		pool_connection_pool[i].info = &(MY_PROCESS_INFO.connection_info[i]);
-		memset(pool_connection_pool[i].info, 0, sizeof(ConnectionInfo));
+		memset(pool_connection_pool[i].info, 0, sizeof(ConnectionInfo) * MAX_NUM_BACKENDS);
 	}
 	return 0;
 }
@@ -154,7 +154,7 @@ POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor, in
 					info = p->info;
 					memset(p, 0, sizeof(POOL_CONNECTION_POOL_SLOT));
 					p->info = info;
-					memset(p->info, 0, sizeof(ConnectionInfo));
+					memset(p->info, 0, sizeof(ConnectionInfo) * MAX_NUM_BACKENDS);
 					POOL_SETMASK(&oldmask);
 					return NULL;
 				}
@@ -201,7 +201,7 @@ void pool_discard_cp(char *user, char *database, int protoMajor)
 	info = p->info;
 	memset(p, 0, sizeof(POOL_CONNECTION_POOL));
 	p->info = info;
-	memset(p->info, 0, sizeof(ConnectionInfo));
+	memset(p->info, 0, sizeof(ConnectionInfo) * MAX_NUM_BACKENDS);
 }
 
 
@@ -254,7 +254,7 @@ POOL_CONNECTION_POOL *pool_create_cp(void)
 	p = oldestp;
 	pool_send_frontend_exits(p);
 
-	pool_debug("discarding old %d th connection. user: %s database: %s",
+	pool_debug("discarding old %zd th connection. user: %s database: %s",
 			   oldestp - pool_connection_pool,
 			   MASTER_CONNECTION(p)->sp->user,
 			   MASTER_CONNECTION(p)->sp->database);
@@ -277,7 +277,7 @@ POOL_CONNECTION_POOL *pool_create_cp(void)
 	info = p->info;
 	memset(p, 0, sizeof(POOL_CONNECTION_POOL));
 	p->info = info;
-	memset(p->info, 0, sizeof(ConnectionInfo));
+	memset(p->info, 0, sizeof(ConnectionInfo) * MAX_NUM_BACKENDS);
 
 	return new_connection(p);
 }
@@ -383,7 +383,7 @@ void pool_backend_timer(void)
 				info = p->info;
 				memset(p, 0, sizeof(POOL_CONNECTION_POOL));
 				p->info = info;
-				memset(p->info, 0, sizeof(ConnectionInfo));
+				memset(p->info, 0, sizeof(ConnectionInfo) * MAX_NUM_BACKENDS);
 			}
 			else
 			{
