@@ -1,11 +1,11 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool-II/pool_rewrite_query.c,v 1.10 2008/07/04 05:27:30 y-mori Exp $
+ * $Header: /cvsroot/pgpool/pgpool-II/pool_rewrite_query.c,v 1.13.2.1 2009/08/22 04:19:49 t-ishii Exp $
  *
- * pgpool: a language independent connection pool server for PostgreSQL 
+ * pgpool: a language independent connection pool server for PostgreSQL
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2008	PgPool Global Development Group
+ * Copyright (c) 2003-2009	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -34,11 +34,12 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend,RewriteQuery
 static void examSelectStmt(Node *node,POOL_CONNECTION_POOL *backend,RewriteQuery *message);
 static char *delimistr(char *str);
 static int direct_parallel_query(RewriteQuery *message);
-static int check_whereClause(Node *where);
 static void initMessage(RewriteQuery *message);
 static void initdblink(ConInfoTodblink *dblink, POOL_CONNECTION_POOL *backend);
 static void analyze_debug(RewriteQuery *message);
 
+
+/* create error message  */
 char *pool_error_message(char *message)
 {
 	String *str;
@@ -48,6 +49,10 @@ char *pool_error_message(char *message)
 	return str->data;
 }
 
+/*
+ *  search DistDefInfo(this info is build in starting process
+ *  and get node id where a query send.
+ */
 static int getInsertRule(ListCell *lc,List *list_t ,DistDefInfo *info,int div_key_num)
 {
 	int loop_counter = 0;
@@ -112,6 +117,10 @@ static int getInsertRule(ListCell *lc,List *list_t ,DistDefInfo *info,int div_ke
 	return node_number;
 }
 
+/*
+ * This function processes the decision whether to
+ * distribute the insert sentence to the node.
+ */
 static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuery *message)
 {
 	RangeVar *table;
@@ -171,14 +180,14 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuer
 		message->rewrite_query = pool_error_message("cannot find target List");
 		return;
 	}
-	
+
 	/* number of target list */
 
 	if(list_t->length == 1 && IsA(lfirst(list_head(list_t)),List))
 	{
 		cell_num = ((List *) lfirst(list_head(list_t)))->length;
-	} 
-	else 
+	}
+	else
 	{
 			/* send  error message to frontend */
 			message->r_code = INSERT_SQL_RESTRICTION;
@@ -206,7 +215,7 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuer
 		}
 
   }
-	else 
+	else
 	{
 		List *list_cols = (List *) insert->cols;
 
@@ -254,13 +263,14 @@ static void examInsertStmt(Node *node,POOL_CONNECTION_POOL *backend, RewriteQuer
 		message->rewrite_query = pool_error_message("cannot get node_id from system db");
 		return;
 	}
-	
+
 	pool_debug("insert node_number =%d",node_number);
 	message->r_code = 0;
 	message->r_node = node_number;
 	message->rewrite_query = nodeToString(node);
 }
 
+/* start of rewriting query */
 static void examSelectStmt(Node *node,POOL_CONNECTION_POOL *backend,RewriteQuery *message)
 {
 	static ConInfoTodblink dblink;
@@ -273,10 +283,11 @@ static void examSelectStmt(Node *node,POOL_CONNECTION_POOL *backend,RewriteQuery
 	message->type = node->type;
 	message->r_code = SELECT_DEFAULT;
 
+  /* do rewrite query */
 	nodeToRewriteString(message,&dblink,node);
 }
 
-
+/* initialize Message */
 static void initMessage(RewriteQuery *message)
 {
 	message->r_code = 0;
@@ -296,6 +307,7 @@ static void initMessage(RewriteQuery *message)
 	message->ret_num = 0;
 }
 
+/* set dblink info */
 static void initdblink(ConInfoTodblink *dblink,POOL_CONNECTION_POOL *backend)
 {
 	dblink->dbname =  MASTER_CONNECTION(backend)->sp->database;
@@ -305,6 +317,7 @@ static void initdblink(ConInfoTodblink *dblink,POOL_CONNECTION_POOL *backend)
 	dblink->password = MASTER_CONNECTION(backend)->con->password;
 }
 
+/* reference of pg_catalog or not */
 int IsSelectpgcatalog(Node *node,POOL_CONNECTION_POOL *backend)
 {
 	static ConInfoTodblink dblink;
@@ -320,7 +333,7 @@ int IsSelectpgcatalog(Node *node,POOL_CONNECTION_POOL *backend)
 
 	initdblink(&dblink,backend);
 
-	if(message.is_pg_catalog) 
+	if(message.is_pg_catalog)
 	{
 		pool_debug("Isselectpgcatalog %d",message.is_pg_catalog);
 		return 1;
@@ -331,6 +344,10 @@ int IsSelectpgcatalog(Node *node,POOL_CONNECTION_POOL *backend)
 	}
 }
 
+/*
+ *  SELECT statement or INSERT statement is special,
+ *  peculiar process is needed in parallel mode.
+ */
 RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNECTION_POOL *backend,RewriteQuery *message)
 {
 	switch(node->type)
@@ -338,6 +355,10 @@ RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNE
 		case T_SelectStmt:
 		{
 			SelectStmt *stmt = (SelectStmt *)node;
+
+			 /* Because "SELECT INTO" cannot be used in a parallel mode,
+			  * the error message is generated and send "ready for query" to frontend.
+			  */
 			if(stmt->intoClause)
 			{
 				pool_send_error_message(frontend, MAJOR(backend), "XX000",
@@ -351,14 +372,21 @@ RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNE
 				break;
 			}
 
+			/*
+			 * The Query is actually rewritten based on analytical information on the Query.
+			 */
 			examSelectStmt(node,backend,message);
 
 			if (message->r_code != SELECT_PGCATALOG &&
 				message->r_code != SELECT_RELATION_ERROR)
 			{
+				/*
+				 * The rewritten Query is transmitted to system db,
+				 * and execution status is received.
+				 */
 				POOL_CONNECTION_POOL_SLOT *system_db = pool_system_db_connection();
-				message->status = OneNode_do_command(frontend, 
-													system_db->con, 
+				message->status = OneNode_do_command(frontend,
+													system_db->con,
 													message->rewrite_query,
 													backend->info->database);
 			}
@@ -367,15 +395,24 @@ RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNE
 				if(TSTATE(backend) == 'T' &&
 				   message->r_code == SELECT_RELATION_ERROR)
 				{
-					pool_debug("pool_rewrite_stmt(select): inside transaction");
+					/*
+					 * In the case of message->r_code == SELECT_RELATION_ERROR and in the transaction,
+					 * Transmit the Query to all back ends, and to abort transaction.
+					 */
+					pool_debug("pool_rewrite_stmt(select): Inside transaction. abort transaction");
 					message->rewrite_query = nodeToString(node);
 					message->status = pool_parallel_exec(frontend,backend,message->rewrite_query,node,true);
-				} 
+				}
 				else
-				{ 
+				{
+					/*
+					 * Ohter cases of message->r_code == SELECT_RELATION_ERROR
+					 * or SELECT_PG_CATALOG,
+					 * Transmit the Query to Master node and receive status.
+					 */
 					pool_debug("pool_rewrite_stmt: executed by Master");
 					message->rewrite_query = nodeToString(node);
-					message->status = OneNode_do_command(frontend, 
+					message->status = OneNode_do_command(frontend,
 														MASTER(backend),
 														message->rewrite_query,
 														backend->info->database);
@@ -384,19 +421,23 @@ RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNE
 			pool_debug("pool_rewrite_stmt: select message_code %d",message->r_code);
 		}
 		break;
-			
+
 		case T_InsertStmt:
+
+		  /* The distribution of the INSERT sentence. */
 			examInsertStmt(node,backend,message);
 
 			if(message->r_code == 0 )
 			{
-				message->status = OneNode_do_command(frontend, 
+				/* send the INSERT sentence */
+				message->status = OneNode_do_command(frontend,
 													CONNECTION(backend,message->r_node),
 													message->rewrite_query,
 													backend->info->database);
 			}
 			else if (message->r_code == INSERT_SQL_RESTRICTION)
 			{
+				/* Restriction case of INSERT sentence */
 				pool_send_error_message(frontend, MAJOR(backend), "XX000",
 										"pgpool2 sql restriction",
 										message->rewrite_query, "", __FILE__,
@@ -404,11 +445,13 @@ RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNE
 
 				if(TSTATE(backend) == 'T')
 				{
-					pool_debug("rewrite_query_stmt(insert): inside transaction");
+					/* In Transaction, send the invalid message to backend to abort this transaction */
+					pool_debug("rewrite_query_stmt(insert): Inside transaction. Abort transaction");
 					message->status = pool_parallel_exec(frontend,backend, "POOL_RESET_TSTATE",node,false);
 				}
 				else
 				{
+					/* return "ready for query" to frontend */
 					pool_send_readyforquery(frontend);
 					message->status=POOL_CONTINUE;
 				}
@@ -426,42 +469,18 @@ RewriteQuery *rewrite_query_stmt(Node *node,POOL_CONNECTION *frontend,POOL_CONNE
 	}
 
 	pool_debug("pool_rewrite_stmt: query rule %d",node->type);
-	
+
 	return message;
 }
 
 #define POOL_PARALLEL "pool_parallel"
 #define POOL_LOADBALANCE "pool_loadbalance"
 
-static int check_whereClause(Node *where)
-{
-	if(IsA(where,SubLink) || IsA(where,RangeSubselect)
-		|| IsA(where,FuncCall) || IsA(where, RowExpr))
-	{
-		return 1;
-	}
-	else if(IsA(where, A_Expr))
-	{
-		A_Expr *expr = (A_Expr *) where;
-		int count = 0;
-
-		if(expr->lexpr)
-		{
-			count = check_whereClause(expr->lexpr);
-		}
-
-		if(expr->rexpr)
-		{
-			count += check_whereClause(expr->rexpr);
-		}
-		return count;
-	}
-	else
-	{	
-		return 0;
-	}
-}
-
+/*
+ * After analyzing query, check the analyze[0]->state.
+ * if the analyze[0]->state ==`P`, this query can be executed
+ * on parallel engine.
+ */
 static int direct_parallel_query(RewriteQuery *message)
 {
 	if(message && message->analyze[0] && message->analyze[0]->state == 'P')
@@ -470,12 +489,14 @@ static int direct_parallel_query(RewriteQuery *message)
 		return 0;
 }
 
+
+/* escape delimiter character */
 static char *delimistr(char *str)
 {
 	char *result;
 	int i,j = 0;
 	int len = strlen(str);
-	
+
 	result = palloc(len -1);
 
 	for(i = 0; i < len; i++)
@@ -483,7 +504,7 @@ static char *delimistr(char *str)
 		char c = (unsigned char) str[i];
 		if((i != 0) && (i != len -1))
 		{
-			if(c=='\'' && (char) str[i+1]=='\'') 
+			if(c=='\'' && (char) str[i+1]=='\'')
 				i++;
 			result[j] = c;
 			j++;
@@ -495,6 +516,7 @@ static char *delimistr(char *str)
 	return result;
 }
 
+/* for debug */
 void analyze_debug(RewriteQuery *message)
 {
 	int analyze_num,i;
@@ -505,9 +527,15 @@ void analyze_debug(RewriteQuery *message)
 		AnalyzeSelect *analyze = message->analyze[i];
 		pool_debug("analyze_debug :select no(%d), last select(%d), last_part(%d), state(%c)",
              analyze->now_select,analyze->last_select,analyze->call_part,analyze->state);
-	}	
+	}
 }
 
+/*
+ * This function checks the KEYWORD(POOL_PARALLEL,POOL_LOADBALANCE)
+ * if the special function(like pool_parallel() or pool_loadbalance())
+ * is used, mark the r_code,is_parallel and is_loadbalance.
+ * In othe cases, It is necessary to analyze the Query.
+ */
 RewriteQuery *is_parallel_query(Node *node, POOL_CONNECTION_POOL *backend)
 {
 	static RewriteQuery message;
@@ -523,15 +551,15 @@ RewriteQuery *is_parallel_query(Node *node, POOL_CONNECTION_POOL *backend)
 
 		stmt = (SelectStmt *) node;
 
-		/* LOADBALANCE OR PARALLEL */
+		/* Check the special function is used in this query*/
 		if (!(stmt->distinctClause || stmt->intoClause ||
 			stmt->fromClause || stmt->groupClause || stmt->havingClause ||
 			stmt->sortClause || stmt->limitOffset || stmt->limitCount ||
-			stmt->lockingClause || stmt->larg || stmt->rarg) && 
+			stmt->lockingClause || stmt->larg || stmt->rarg) &&
 			(n = lfirst(list_head(stmt->targetList))) && IsA(n, ResTarget))
 		{
 			ResTarget *target = (ResTarget *) n;
-		
+
 			if (target->val && IsA(target->val, FuncCall))
 			{
 				FuncCall *func = (FuncCall *) target->val;
@@ -540,30 +568,30 @@ RewriteQuery *is_parallel_query(Node *node, POOL_CONNECTION_POOL *backend)
 					Node *func_args = (Node *) lfirst(list_head(func->args));
 					message.rewrite_query = delimistr(nodeToString(func_args));
 
-					/* PARALLEL */
+					/* pool_parallel() is used in this query */
 					if(strcmp(strVal(lfirst(list_head(func->funcname))),
 						   POOL_PARALLEL) == 0)
 					{
 						message.r_code = SEND_PARALLEL_ENGINE;
 						message.is_parallel = true;
 						message.is_loadbalance = false;
-						pool_debug("can pool_parallel_exec %s",message.rewrite_query);	
+						pool_debug("can pool_parallel_exec %s",message.rewrite_query);
 						return &message;
-					} 
-					else /* LOADBALANCE */ 
+					}
+					else /* pool_loadbalance() is used in this query */
 					if(strcmp(strVal(lfirst(list_head(func->funcname))),
 						   						POOL_LOADBALANCE) == 0)
 					{
 						message.r_code = SEND_LOADBALANCE_ENGINE;
 						message.is_loadbalance = true;
 						message.is_parallel = false;
-						pool_debug("can loadbalance_mode %s",message.rewrite_query);	
+						pool_debug("can loadbalance_mode %s",message.rewrite_query);
 						return &message;
 					}
-				} 
+				}
 			}
 		}
-		
+
     /* ANALYZE QUERY */
 		message.r_code = SELECT_ANALYZE;
 		message.is_loadbalance = true;
@@ -586,21 +614,25 @@ RewriteQuery *is_parallel_query(Node *node, POOL_CONNECTION_POOL *backend)
 			return &message;
 		}
 
+		/* Analyzing Query Start */
 		analyze_debug(&message);
 
-		/* PARALLEL*/
+		/* After the analyzing query,
+		 * this query can be executed as parallel exec, is_parallel flag is turned on
+		 */
 		direct_ok = direct_parallel_query(&message);
 		if(direct_ok == 1)
 		{
 			message.rewrite_query = nodeToString(node);
 			message.is_parallel = true;
 			message.is_loadbalance = false;
-			pool_debug("can pool_parallel_exec %s",message.rewrite_query);	
+			pool_debug("can pool_parallel_exec %s",message.rewrite_query);
 			return &message;
 		}
 	}
 	else if (IsA(node, CopyStmt))
 	{
+		/* For Copy Statement, check the table name, mark the is_parallel flag. */
 		CopyStmt *stmt = (CopyStmt *)node;
 
 		if (stmt->is_from == FALSE && stmt->filename == NULL)
